/**
 * Date:            Time:       User:               Description:
 * 14.03.2006       20:40       Smart Max           creation of the file INode.java
 *
 */
package org.max.trie;

import java.util.Set;
import java.util.List;

/**
 * INode
 * <p>
 * To serialize the Patricia trie, the nodes must be serializable.
 * <p>
 * The <tt>INode</tt> interface provides only get methods for node data (depth, parent, children, ...);
 * to update the data use {@link IWritableNode} interface
 *
 * @author  Maxim Podkolzine
 */
public interface INode {
    /**
     * Returns the list of references of the node.
     * <p>
     * Every node of Patricia trie can have a special marker (reference), which indicates from what document
     * corresponding string was added. Only using the reference to the source document, the verification
     * of searching operation can be performed (see {@link IPatriciaTrie}). Reference can have also the
     * coordinates of the string inside the document.
     * <p>
     * For example, consider the set of documents "doc1", "doc2", "doc3". From first document we will
     * add strings "castle" and "foreign"; from second: "cat", "castiron" and "forever" and from the third:
     * "cast" and "castle".
     * <p>
     * After trie constructing, it will have 9 nodes, 6 of which will have references. Obviously, that every
     * leaf node have a marker, but not only. Note, that node that corresponds to key "cast" has a reference
     * and is not leaf; also note that node "castle" have two references (on "doc1" and "doc3").
     * <p>
     * By contract, the node that has no references, must return <tt>null</tt>
     *
     * @return                                      the list of references
     */
    public List<IReference> getReferences();

    /**
     * Returns the depth of the node.
     * <p>
     * Since not all symbols are held on edges of the trie (see structure of {@link IPatriciaTrie}),
     * depth of node acquires the great importance. While searching in a trie, only one symbol on each
     * edge is compared (see {@link IPatriciaTrie#get(java.util.List<org.max.trie.ISymbol>)}. More
     * precisely, the symbols with same offset (in a trie it is depth) are compared.
     * <p>
     * The depth of the node means the number of symbols needed to get to this node from root.
     * The root node always have zero depth, its child can have depth <tt>1</tt> (if an edge has exactly
     * one symbol), or <tt>2</tt> (if it has two symbols), and so on.
     *
     * @return                                      the depth of the node
     */
    public int getDepth();

    /**
     * Returns the parent of the node.
     * <p>
     * The parent is not defined for the root node and by contract the root always returns <tt>null</tt>.
     * For other nodes the parent is defined, but method can still return <tt>null</tt>. The point is that
     * trie can be stored not as a whole, but piece-meal. This means that the child of some node can
     * be stored on disk and in this case the child doesn't know about the parent. It is done for
     * independence of blocks which divide trie from each other.
     * <p>
     * But it is guaranteed that if parent is not <tt>null</tt>, then it has a child with corresponding
     * label on edge. It is also guaranteed that for not-deserialized trie (see
     * {@link org.max.trie.impl.PatriciaTrieSerializer}) the parent exists for all not-root nodes.
     *
     * @return                                      the parent node
     */
    public INode getParent();

    /**
     * Returns the symbol on the incoming edge of the node.
     * <p>
     * The incoming edge is not defined for root node and by contract the root always returns <tt>null</tt>.
     * For other nodes the incoming edge is defined and (in contrast with {@link #getParent()}) this
     * method always returns it. It is guaranteed that the child by label <tt>s</tt>, will have incoming
     * symbol <tt>s</tt>. In other words, for every <tt>INode node</tt> and <tt>ISymbol label</tt>
     * <pre>
     * INode child = node.getChild(label);
     * if (child != null) {
     *     System.out.println(child.getIncomingSymbol() == label);
     * }
     * </pre>
     * this piece of code will never print <tt>false</true>.
     *
     * @return                                      the incoming symbol
     * @see     #getOutcomingSymbols()
     */
    public ISymbol getIncomingSymbol();

    /**
     * Returns the set of symbols on outcoming edges of the node.
     * <p>
     * For every node of the trie method returns the set of symbols (possibly empty, but not <tt>null</tt>).
     * And for each symbol it is guaranteed that there exists the child connected with this node by
     * an edge with such symbol. Also it is guaranteed that every outcoming symbol will be contained
     * in this set.
     * <p>
     * Also this method coordinated with {@link #hasLabel(ISymbol)} method in following sense:
     * symbol <tt>s</tt> is contained in <tt>node.getOutcomingSymbols()</tt> if and only if
     * <tt>node.hasLabel(s) == true</tt>.
     *
     * @return                                      the set of outcoming symbols
     * @see     #hasLabel(ISymbol)
     * @see     #getChild(ISymbol)
     */
    public Set<ISymbol> getOutcomingSymbols();

    /**
     * Returns whether this node has specified label among outcoming edges.
     * <p>
     * Method returns <tt>true</tt> if and only if for some child node there exists an
     * edge (between this node and child) with such label. It is guaranteed that each label <tt>l</tt>
     * that satisfy <tt>node.hasLabel(l)</tt> must lie in <tt>node.getOutcomingSymbols()</tt>
     *
     * @param   label                               the symbol
     * @return                                      whether the node has such outcoming symbol
     * @see     #getOutcomingSymbols()
     * @see     #getChild(ISymbol)
     */
    public boolean hasLabel(ISymbol label);

    /**
     * Returns the child of node by specified label.
     * <p>
     * Method returns the node, if it has the child with specified label on edge between them.
     * And in this case, the child will have expected incoming symbol and possibly parent
     * (see {@link #getIncomingSymbol()} and {@link #getParent()}).
     * <p>
     * It is guaranteed that method will recieve the child (if it exists), even if new block
     * uploading will be needed, and return it.
     *
     * @param   label                               the symbol
     * @return                                      the child node if it exists, and <tt>null</tt> otherwise
     * @see     #getOutcomingSymbols()
     * @see     #hasLabel(ISymbol) 
     */
    public INode getChild(ISymbol label);
}
