/**
 * Date:            Time:       User:               Description:
 * 19.04.2006       11:57       Smart Max           creation of the file PatriciaTrieSerializer.java
 *
 */
package org.max.trie.impl;

import static org.max.NativeSystem.*;

import org.max.trie.IPatriciaTrie;

import java.util.List;
import java.io.*;

/**
 * PatriciaTrieSerializer
 *
 * @author  Maxim Podkolzine
 */
public class PatriciaTrieSerializer {
    // Data

    /**
     * Holds the Patricia trie to be serialized
     */
    private IPatriciaTrie myTrie;

    /**
     * Holds the directory, in which the trie would be serialized
     */
    private String myDirectory;




    // Constructor

    /**
     * Creates a serializer of specified trie with specified directory name
     *
     * @param   trie                                the trie to be serialized
     * @param   directory                           the destination directory name
     */
    public PatriciaTrieSerializer(IPatriciaTrie trie, String directory) {
        myTrie = trie;
        myDirectory = directory;
    }




    // Main interface

    /**
     * Serializes the trie. After serialization the trie will be stored in several files (possibly one),
     * each of which will contain not more than <tt>blockSize</tt> nodes.
     * <p>
     * In fact, method divides a trie on blocks, using PatriciaTrieDivider and then
     * serialize each block into the directory. Also it creates the file "meta.dat" and writes the
     * root block name in it.
     * <p>
     * Method does not write the map of symbols on edges, so after deserialization no further addings
     * would be admissible. Use methods {@link #writeFullTrie(int)} and {@link #writeFullTrie()} to preserve
     * the ability to modify the trie.
     *
     * @param   blockSize                           the maximal block size
     * @throws  IOException                         if I/O errors occur
     */
    public void writeTrie(int blockSize) throws IOException {
        PatriciaTrieDivider divider = new PatriciaTrieDivider();
        PatriciaTrieDivider.setBlockSize(blockSize);
        List<Block> blocks = divider.divide(myTrie, myDirectory);

        for (Block block : blocks) {
            ObjectOutput output = new ObjectOutputStream(newOutputStream(myDirectory + Separator + block.getName()));
            output.writeObject(block);
            output.close();
        }

        Writer writer = newWriter(myDirectory + Separator + "meta.dat");
        writer.write(blocks.get(blocks.size() - 1).getName());
        writer.close();
    }

    /**
     * Serializes the trie. After serialization the trie will be stored in several files (possibly one),
     * each of which will contain not more than <tt>50</tt> nodes.
     * <p>
     * This method is strictly equivalent to {@link #writeTrie(int)} with parameter <tt>blockSize == 50</tt>.
     *
     * @throws  IOException                         if I/O errors occur
     */
    public void writeTrie() throws IOException {
        writeTrie(50);
    }

    // Methods to write full trie

    /**
     * Saves the map of symbols on edges to the file "triemap.dat"
     *
     * @throws  IOException                         if I/O errors occur
     */
    private void saveMap() throws IOException {
        ObjectOutput output = new ObjectOutputStream(newOutputStream(myDirectory + Separator + "triemap.dat"));
        output.writeObject(((PatriciaTrie) myTrie).getMap());
        output.close();
    }

    /**
     * Serializes the trie fully. After serialization the trie will be stored in several files (possibly one),
     * each of which will contain not more than <tt>blockSize</tt> nodes.
     * <p>
     * Method does strictly the same as {@link #writeTrie(int)} and also saves the map of symbols on edges.
     * Thus, the trie would be changeable after deserialization.
     *
     * @param   blockSize                           the maximal block size
     * @throws  IOException                         if I/O errors occur
     */
    public void writeFullTrie(int blockSize) throws IOException {
        writeTrie(blockSize);
        saveMap();
    }

    /**
     * Serializes the trie fully. After serialization the trie will be stored in several files (possibly one),
     * each of which will contain not more than <tt>50</tt> nodes.
     * <p>
     * This method is strictly equivalent to {@link #writeFullTrie(int)} with parameter <tt>blockSize == 50</tt>.
     *
     * @throws  IOException                         if I/O errors occur
     */
    public void writeFullTrie() throws IOException {
        writeTrie();
        saveMap();
    }
}
