/**
 * Date:            Time:       User:               Description:
 * 18.04.2006       19:28       Smart Max           creation of the file IndexedSet.java
 *
 */
package org.max.util;

import java.util.*;
import java.io.Serializable;
import java.io.IOException;
import java.io.ObjectInputStream;

/**
 * IndexedSet
 * <p>
 * The indices in <tt>myElements</tt> and in <tt>myIndices</tt> are coordinated.
 *
 * @author  Maxim Podkolzine
 */
public class IndexedSet<T> implements Iterable<T>, Serializable {
    private static final long serialVersionUID = 32495684782109834L;

    /**
     * Reads the <tt>IndexedSet</tt> instance from specified input stream.
     * <p>
     * Method reads the map {@link #myIndices} and then constructs the list {@link #myElements}
     * of elements in right order.
     *
     * @param   input                               the input stream
     * @throws  IOException                         if I/O errors occur
     * @throws  ClassNotFoundException              if the class of a serialized object can not be found
     */
    @SuppressWarnings({"unchecked"})
    private void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException {
        input.defaultReadObject();

        myElements = new ArrayList<T>(myIndices.size());
        Object[] elementsArray = new Object[myIndices.size()];

        for (T element : myIndices.keySet()) {
            int index = myIndices.get(element);
            elementsArray[index] = element;
        }

        for (Object element : elementsArray) {
            myElements.add((T) element);
        }
    }




    // Data

    /**
     * Holds the list of elements.
     * <p>
     * This list is needed to get element by index.
     * <p>
     * The field is transient because it can be determined by indices map (see {@link #myIndices}).
     */
    private transient List<T> myElements;

    /**
     * Holds the map from elements to integers.
     * <p>
     * This map is needed for fast getting index of element. 
     */
    private Map<T, Integer> myIndices;




    // Constructor

    /**
     * Creates an empty indexed set
     */
    public IndexedSet() {
        myIndices = new HashMap<T, Integer>();
        myElements = new ArrayList<T>();
    }




    // Main interface

    /**
     * Adds the element to the set.
     * <p>
     * If such element already exists in set, method does nothing. Otherwise, the new element gets new index
     * (in fact, this index equals to the size of map before adding) and is added to the set. In any case, the
     * index of element is returned.
     *
     * @param   element                             the element to be added
     * @return                                      the index of element
     * @see     #get(int)
     * @see     #getIndex(Object)
     */
    public int add(T element) {
        Integer index = myIndices.get(element);

        if (index != null) {
            return index;
        } else {
            myIndices.put(element, myElements.size());
            myElements.add(element);
            return myElements.size() - 1;
        }
    }

    /**
     * Returns the element by its index.
     * <p>
     * It is guaranteed the constant execution time of request.
     *
     * @param   index                               an index of element
     * @return                                      the requested element
     * @see     #getIndex(Object)
     */
    public T get(int index) {
        return myElements.get(index);
    }

    /**
     * Returns the index of specified element.
     * <p>
     * It is not guaranteed the constant execution time of request, but it will be not slower than
     * straight search in list. The exact time of work depends on elements (more precisely, on hash
     * code distribution of elements).
     *
     * @param   element                             the sought element
     * @return                                      the index of an element, or <tt>-1</tt> if element
     *                                              was not found
     * @see     #get(int)
     */
    public int getIndex(T element) {
        Integer index = myIndices.get(element);

        if (index == null) {
            return -1;
        } else {
            return index;
        }
    }

    /**
     * Returns whether the specified element is contained in set
     *
     * @param   element                             the sought element
     * @return                                      true iff the set contains the element
     */
    public boolean contains(T element) {
        return myIndices.containsKey(element);
    }

    /**
     * Returns an iterator over the elements in the set in proper sequence
     *
     * @return                                      an iterator over the elements in this list in proper sequence
     */
    public Iterator<T> iterator() {
        return myElements.iterator();
    }

    /**
     * Returns the number of elements in the set
     *
     * @return                                      the number of elements in the set
     */
    public int size() {
        return myElements.size();
    }
}
