/**
 * Date:            Time:       User:               Description:
 * 04.05.2006       21:49       Smart Max           creation of the file Index.java
 *
 */
package org.max.xml.index;

import static org.max.NativeSystem.*;

import net.sf.xanswer.indexing.IndexApi;
import net.sf.xanswer.indexing.BulkLoader;
import net.sf.xanswer.indexing.IndexException;

import java.io.InputStream;

/**
 * Index
 *
 * @author  Maxim Podkolzine
 */
public class Index implements IndexApi, BulkLoader {
    // Data

    /**
     * Holds the location of the Index Fabric
     */
    private String myLocation;

    /**
     * Holds the index of added document
     */
    private int myDocumentsIndex;

    /**
     * Holds whether the Index Fabric is modifiable
     */
    private boolean myModifiable;

    /**
     * Holds the Index Fabric
     */
    private IndexFabric myIndexFabric;

    /**
     * Holds the builder of Index Fabric from XML
     */
    private XMLBuilder myBuilder;




    // Main interface

    /**
     * Creates an Index Fabric, using specified location
     *
     * @param   location                            the destination location
     * @throws  IndexException                      if index creation fails
     * @see     #open()
     */
    public void create(String location) throws IndexException {
        myLocation = location;
    }

    /**
     * Open an existed Index Fabric.
     * <p>
     * At first, the location of index is specified using method <tt>create(String)</tt>, and
     * after this an index can be uploaded. If location is not found, this method creates a new
     * Index Fabric with given location.
     * <p>
     * By default, if the previously created Index Fabric was loaded, it becomes unmodifiable;
     * however, a newly created Index Fabric becomes modifiable.
     *
     * @throws  IndexException                      if errors of Index Fabric uploading occur
     * @see     #create(String)
     * @see     #close()
     */
    public void open() throws IndexException {
        if (!exists(myLocation)) {
            myBuilder = new XMLBuilder();
            myModifiable = true;
        } else {
            myIndexFabric = new IndexFabricDeserializer().readIndexFabric(myLocation);
            myModifiable = false;
        }
    }

    /**
     * Closes the Index Fabric.
     * <p>
     * If this Index Fabric was modifiable, it is serialized to the location, indicated in {@link #create(String)}
     * method. The unmodifiable Index Fabric is not serialized.
     * After this, memory occupied by Index Fabric is released.
     *
     * @throws  IndexException                      if errors of Index Fabric saving occur
     */
    public void close() throws IndexException {
        if (myModifiable) {
            createDirectory(myLocation);
            IndexFabricSerializer serializer = new IndexFabricSerializer(myIndexFabric, myLocation);
            serializer.writeFabric();
        }

        myIndexFabric = null;
        myBuilder = null;
    }

    /**
     * Loads the document from specified input stream.
     * <p>
     * The loaded document receives default identifier. It is guaranteed that all documents
     * that are added using this method will get different identifiers. But it can coincide with
     * one if the document was added using {@link #load(String, InputStream)} method.
     * <p>
     * If Index Fabric is unmodifiable, an <tt>UnsupportedOperationException</tt> raises.
     * In case of errors while document opening or parsing, method throws <tt>IndexException</tt>.
     *
     * @param   input                               the input stream
     * @return                                      corresponding identifier of added document
     * @throws  IndexException                      if errors while document adding occur
     * @see     #load(String, InputStream)
     */
    public String load(InputStream input) throws IndexException {
        if (!myModifiable) {
            throw new UnsupportedOperationException("Index can not load new XML documents");
        }

        String docID = "document #" + (myDocumentsIndex++);
        myBuilder.addXMLDocument(docID, input);

        return docID;
    }

    /**
     * Loads the document with specified identifier from specified input stream.
     * <p>
     * If Index Fabric is unmodifiable, an <tt>UnsupportedOperationException</tt> raises.
     * In case of errors while document opening or parsing, method throws <tt>IndexException</tt>.
     *
     * @param   docID                               the document identifier
     * @param   input                               the input stream
     * @throws  IndexException                      if errors while document adding occur
     * @see     #load(InputStream)
     */
    public void load(String docID, InputStream input) throws IndexException {
        if (!myModifiable) {
            throw new UnsupportedOperationException("Index can not load new XML documents");
        }

        myBuilder.addXMLDocument(docID, input);
    }
}
