/**
 * Date:            Time:       User:               Description:
 * 16.03.2006       13:22       Smart Max           creation of the file Symbol.java
 *
 */
package org.max.xml.index;

import static org.w3c.dom.Node.*;

import org.max.trie.ISymbol;

import java.io.Serializable;

/**
 * Class Symbol.
 * Using XMLBuilder to construct PatriciaTrie, there are few instances of class Symbol. This is archieved
 * by bidirectional maps that remember what symbols are already created. In ordinary situation, there
 * are ~100 text symbols (letters 'a'..'z', 'A'..'Z', digits '0'..'9', punctuation marks, brackets, and etc),
 * and order of 100 tag and attribute symbols.
 * In this case, serialization of class Symbol does not need to be compressed. For example, if full size of trie
 * is ~1 Mbyte, the total size of all symbols would be ~1 Kbyte.
 *
 * @author  Maxim Podkolzine
 */
class Symbol implements ISymbol, Serializable {
    private static final long serialVersionUID = 30985739853245009L;




    // Data

    /**
     * Holds the string name of symbol
     */
    private String myName;

    /**
     * Holds the type of symbol.
     * <p>
     * Typically, it is one of these: tag (or element), attribute or text.
     */
    private int myType;




    // Constructor

    /**
     * Creates a new symbol with specified name and type
     *
     * @param   name                                the symbol name
     * @param   type                                the symbol type
     */
    public Symbol(String name, int type) {
        myName = name;
        myType = type;
    }

    /**
     * Creates a new symbol with specified name and type
     *
     * @param   ch                                  the character which represents the name of symbol
     * @param   type                                the symbol type
     */
    public Symbol(char ch, int type) {
        myName = String.valueOf(ch);
        myType = type;
    }

    /**
     * Creates the symbol of element (tag) type
     *
     * @param   name                                the symbol name
     */
    public Symbol(String name) {
        this(name, ELEMENT_NODE);
    }

    /**
     * Creates a new symbol with specified name of element (tag) type
     *
     * @param   ch                                  the character which represents the name of symbol
     */
    public Symbol(char ch) {
        this(ch, ELEMENT_NODE);
    }




    // Main interface

    /**
     * Indicates whether some other object is "equal to" this one.
     * <p>
     * Method returns <tt>true</tt> if and only if <tt>o</tt> is instance of <tt>Symbol</tt>,
     * and has the same name and type.
     *
     * @param   o                                   an object to compare with
     * @return                                      whether a specified object is equal to this one
     */
    public boolean equals(Object o) {
        if (!(o instanceof Symbol)) {
            return false;
        }

        final Symbol that = (Symbol) o;
        return (myType == that.myType && myName != null ? myName.equals(that.myName) : that.myName == null);
    }

    /**
     * Returns a hash code value for the object
     *
     * @return                                      a hash code value for the object
     */
    public int hashCode() {
        return 29 * myType + (myName != null ? myName.hashCode() : 0);
    }

    /**
     * Returns a string representation of symbol
     *
     * @return                                      a string representation of symbol
     */
    public String toString() {
        if (myType == ATTRIBUTE_NODE) {
            return myName + "'";
        } else {
            return myName;
        }
    }
}
