/**
 * Date:            Time:       User:               Description:
 * 16.03.2006       15:46       Smart Max           creation of the file XMLRetriever.java
 *
 */
package org.max.xml.index;

import org.max.trie.ISymbol;
import org.max.trie.IPatriciaTrie;
import org.max.trie.INode;
import org.max.util.BidirectionalMap;

import java.util.List;
import java.util.ArrayList;

/**
 * XMLRetriever.
 * Retriever seeks in IndexFabric
 * <p>
 * Holds all direct references of IndexFabric for code shortening
 *
 * @author  Maxim Podkolzine
 */
public class XMLRetriever {
    // Data

    /**
     * Holds the Patricia trie
     */
    private IPatriciaTrie myTrie;

    /**
     * Holds the map of text symbols
     */
    private final BidirectionalMap<String, ISymbol> myTextMap;

    /**
     * Holds the map of symbols corresponding to tags in XML files
     */
    private final BidirectionalMap<String, ISymbol> myTagMap;

    /**
     * Holds the map of symbols corresponding to attributes in XML files
     */
    private final BidirectionalMap<String, ISymbol> myAttrMap;




    // Constructor

    /**
     * Creates the retriever over specified Index Fabric
     *
     * @param   indexFabric                         the Index Fabric
     */
    public XMLRetriever(IndexFabric indexFabric) {
        myTrie = indexFabric.getTrie();
        myTextMap = indexFabric.getTextMap();
        myTagMap = indexFabric.getTagMap();
        myAttrMap = indexFabric.getAttrMap();
    }




    // Main interface

    /**
     * Returns the node of Patricia trie that corresponds to specified key path. The key is
     * assigned as list of tag names, after this the attribute name and finally the text value.
     * The attribute name can be omitted using empty <tt>attribute</tt> passing.
     * The text value can be omitted using empty <tt>value</tt> passing.
     * <p>
     * Method transforms each tag name into a designator using tag designators map, appends the
     * designator, corresponding to attribute name (if it is not empty), and all symbols of text
     * using text map. If any of symbols is not found in map, <tt>IncorrectRequestException</tt>
     * exception raises. Else the acquired key is sought in Patricia trie and the result of this
     * search is returned.
     * <p>
     * Parameters tags, attribute and value are considered to be not <tt>null</tt>. Otherwise
     * the <tt>NullPointerException</tt> is raised.
     *
     * @param   tags                                the list of tag names
     * @param   attribute                           the attribute name (or empty string, if attribute
     *                                              is omitted)
     * @param   value                               the text or attribute value (or empty string, if text
     *                                              is omitted)
     * @return                                      the corresponding node of the trie, or
     *                                              <tt>null</tt> if the key was not found
     * @throws  NullPointerException                if one of parameters is <tt>null</tt>
     * @throws  IncorrectRequestException           if one of input symbols is not recognized in
     *                                              Index Fabric map
     * @see     #retrieveRawPath(List<String>, String)
     * @see     #retrieveRawPath(List<String>)
     * @see     IPatriciaTrie#get(List<ISymbol>)
     */
    public INode retrieveRawPath(List<String> tags, String attribute, String value)
            throws IncorrectRequestException {
        if (tags == null || attribute == null || value == null) {
            throw new NullPointerException("Incorrect request. One of parameters is null");
        }

        List<ISymbol> key = new ArrayList<ISymbol>(tags.size() + value.length());

        for (String tag : tags) {
            ISymbol symbol = myTagMap.getB(tag);
            if (symbol == null) {
                throw new IncorrectRequestException("Element " + tag + " not found in dictionary");
            }
            key.add(symbol);
        }

        if (attribute.length() > 0) {
            ISymbol symbol = myAttrMap.getB(attribute);
            if (symbol == null) {
                throw new IncorrectRequestException("Attribute " + attribute + " not found in dictionary");
            }
            key.add(symbol);
        }

        for (int i = 0; i < value.length(); i++) {
            ISymbol symbol = myTextMap.getB(String.valueOf(value.charAt(i)));
            if (symbol == null) {
                throw new IncorrectRequestException("Text symbol " + value.charAt(i) + " not found in dictionary");
            }
            key.add(symbol);
        }

        return myTrie.get(key);
    }

    /**
     * Returns the node of Patricia trie that corresponds to specified key path.
     * <p>
     * Method is strictly equivalent to {@link #retrieveRawPath(List<String>, String, String)} with
     * <tt>attribute</tt> parameter equal to <tt>""</tt>.
     *
     * @param   tags                                the list of tag names
     * @param   value                               the text or attribute value (or empty string, if text
     *                                              is omitted)
     * @return                                      the corresponding node of the trie, or
     *                                              <tt>null</tt> if the key was not found
     * @throws  NullPointerException                if one of parameters is <tt>null</tt>
     * @throws  IncorrectRequestException           if one of input symbols is not recognized in
     *                                              Index Fabric map
     * @see     #retrieveRawPath(List<String>, String, String)
     * @see     #retrieveRawPath(List<String>)
     */
    public INode retrieveRawPath(List<String> tags, String value) throws IncorrectRequestException {
        return retrieveRawPath(tags, "", value);
    }

    /**
     * Returns the node of Patricia trie that corresponds to specified key path.
     * <p>
     * Method is strictly equivalent to {@link #retrieveRawPath(List<String>, String, String)} with
     * <tt>attribute</tt> and <tt>value</tt> parameters equal to <tt>""</tt>.
     *
     * @param   tags                                the list of tag names
     * @return                                      the corresponding node of the trie, or
     *                                              <tt>null</tt> if the key was not found
     * @throws  NullPointerException                if one of parameters is <tt>null</tt>
     * @throws  IncorrectRequestException           if one of input symbols is not recognized in
     *                                              Index Fabric map
     * @see     #retrieveRawPath(List<String>, String, String)
     * @see     #retrieveRawPath(List<String>, String)
     */
    public INode retrieveRawPath(List<String> tags) throws IncorrectRequestException {
        return retrieveRawPath(tags, "", "");
    }
}
