/**
 * Date:            Time:       User:               Description:
 * 18.04.2006       17:41       Smart Max           creation of the file BlockTest.java
 *
 */
package org.max.trie.impl;

import static org.max.trie.impl.TestUtil.*;
import static org.max.NativeSystem.*;

import junit.framework.TestCase;

import java.io.*;
import java.util.Queue;
import java.util.LinkedList;

import org.max.xml.index.XMLBuilder;
import org.max.trie.IPatriciaTrie;
import org.max.trie.INode;
import org.max.trie.ISymbol;
import net.sf.xanswer.indexing.IndexException;


/**
 * BlockTest
 */
public class BlockTest extends TestCase {
    private void addAllTrie(Block block, IPatriciaTrie trie) {
        INode root = trie.getRoot();
        assert(root != null);

        Queue<INode> q = new LinkedList<INode>();
        q.add(root);
        while (!q.isEmpty()) {
            INode node = q.poll();
            block.addNode(node);

            for (ISymbol label : node.getOutcomingSymbols()) {
                INode child = node.getChild(label);
                block.addEdge(node, child);
                q.add(child);
            }
        }

        assert(trie.size() == block.size());
    }

    public void test_Simple() throws IOException, IndexException {
        Block block = new Block("test" + Separator + "$name #007");

        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "doc1.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        addAllTrie(block, trie);

        ObjectOutput output = new ObjectOutputStream(newOutputStream(block.getName()));
        output.writeObject(block);
        output.close();
    }

    /**
     * Test on time optimization. The main piece of code is BFS in trie and adding nodes and edges to the block.
     * While Block holds nodes and edges in the list, the method works 67.5 seconds.
     * If Nodes are sought in list using lastIndexOf, the time would be 47.5 seconds.
     * After introducing of IndexedSet, it became 6-6.5 seconds.
     *
     * @throws java.io.IOException
     */
    public void test_TimeOptimization() throws IOException, IndexException {
        Block block = new Block("test" + Separator + "doc1 + mscorlib");

        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "doc1.xml");
        xmlBuilder.addXMLDocument("xml" + Separator + "mscorlib.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        addAllTrie(block, trie);

        ObjectOutput output = new ObjectOutputStream(newOutputStream(block.getName()));
        output.writeObject(block);
        output.close();
    }

    public void test_SaveAndUpload1() throws IOException, ClassNotFoundException, IndexException {
        Block block = new Block("test" + Separator + "test_SaveAndUpload1");

        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "doc1.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        addAllTrie(block, trie);

        ObjectOutput output = new ObjectOutputStream(newOutputStream(block.getName()));
        output.writeObject(block);
        output.close();

        ObjectInput input = new ObjectInputStream(newInputStream(block.getName()));
        Block block2 = (Block) input.readObject();
        input.close();

        assertEquals(block.size(), block2.size());

        Node root1 = (Node) trie.getRoot();
        Node root2 = (Node) block2.getSubTrie();
        assertEqualSubTries(root1, root2);
    }

    public void test_SaveAndUpload2() throws IOException, ClassNotFoundException, IndexException {
        Block block = new Block("test" + Separator + "test_SaveAndUpload2");

        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "unicode.xml");
        xmlBuilder.addXMLDocument("xml" + Separator + "mscorlib.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        addAllTrie(block, trie);

        ObjectOutput output = new ObjectOutputStream(newOutputStream(block.getName()));
        output.writeObject(block);
        output.close();

        ObjectInput input = new ObjectInputStream(newInputStream(block.getName()));
        Block block2 = (Block) input.readObject();
        input.close();

        assertEquals(block.size(), block2.size());

        Node root1 = (Node) trie.getRoot();
        Node root2 = (Node) block2.getSubTrie();
        assertEqualSubTries(root1, root2);
    }

    public void test_SaveAndUploadAll() throws IOException, ClassNotFoundException, IndexException {
        Block block = new Block("test" + Separator + "test_SaveAndUploadAll");

        XMLBuilder xmlBuilder = new XMLBuilder();
        File xmlDir = new File("xml" + Separator);
        String[] fileNames = xmlDir.list();
        for (String name : fileNames) {
            if (!name.equals("random")) {
                xmlBuilder.addXMLDocument(xmlDir.getAbsolutePath() + Separator + name);
            }
        }

        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        addAllTrie(block, trie);

        ObjectOutput output = new ObjectOutputStream(newOutputStream(block.getName()));
        output.writeObject(block);
        output.close();

        ObjectInput input = new ObjectInputStream(newInputStream(block.getName()));
        block = (Block) input.readObject();
        input.close();

        assertEquals(trie.size(), block.size());

        Node root1 = (Node) trie.getRoot();
        Node root2 = (Node) block.getSubTrie();
        assertEqualSubTries(root1, root2);
    }
}
